/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package gxstrings

import (
	"testing"
)

import (
	"github.com/stretchr/testify/assert"
)

func Test_RegSplit(t *testing.T) {
	strings := RegSplit("dubbo://123.1.2.1;jsonrpc://127.0.0.1;registry://3.2.1.3?registry=zookeeper", "\\s*[;]+\\s*")
	assert.Len(t, strings, 3)
	assert.Equal(t, "dubbo://123.1.2.1", strings[0])
	assert.Equal(t, "jsonrpc://127.0.0.1", strings[1])
	assert.Equal(t, "registry://3.2.1.3?registry=zookeeper", strings[2])
}

func TestIsMatchPattern(t *testing.T) {
	assert.Equal(t, true, IsMatchPattern("*", "value"))
	assert.Equal(t, true, IsMatchPattern("", ""))
	assert.Equal(t, false, IsMatchPattern("", "value"))
	assert.Equal(t, true, IsMatchPattern("value", "value"))
	assert.Equal(t, true, IsMatchPattern("v*", "value"))
	assert.Equal(t, true, IsMatchPattern("*ue", "value"))
	assert.Equal(t, true, IsMatchPattern("*e", "value"))
	assert.Equal(t, true, IsMatchPattern("v*e", "value"))
	assert.Equal(t, true, IsMatchPattern("val*e", "value"))
}
